export interface RofOptions extends Intl.NumberFormatOptions {
    integerThreshold?: number;
}
export interface RofResolvedOptions extends Intl.NumberFormatOptions {
    integerThreshold: number;
    minimumSignificantDigits: number;
    maximumSignificantDigits: number;
}
export declare class Rof {
    private intNumberFormat;
    private decNumberFormat;
    private options;
    constructor(locales?: string | string[], options?: RofOptions);
    /**
     * Formats a value as an integer (using `formatInteger`)
     * or a decimal (using `formatDecimal`) if the value is not an integer.
     *
     * @param x  The number to format.
     * @returns  A string with a formatted representation of the given number.
     */
    format: (x: number) => string;
    /**
     * Formats a value as an integer
     * or float (using `formatFloat`) for large values.
     *
     * @param x  The number to format.
     * @returns  A string with a formatted representation of the given number.
     */
    formatInteger: (x: number) => string;
    /**
     * Formats a value as a decimal
     * or float (using `formatFloat`) for large or small values.
     *
     * @param x  The number to format.
     * @returns  A string with a formatted representation of the given number.
     */
    formatDecimal: (x: number) => string;
    /**
     * Formats a value as a float (exponential) using the rule of four to determine the precision.
     *
     * @param x  The number to format.
     * @returns  A string with a formatted representation of the given number.
     */
    formatFloat: (x: number) => string;
    /**
     * Formats a value as a decimal using the strict rule of four to determine the precision.
     *
     * rule of four:
     * * uses three decimal places for ratios in the range 0.040-0.399
     * * two decimals for 0.40-3.99, one decimal for 4.0-39.9, etc
     *
     * @param x  The number to format.
     * @returns  A string with a formatted representation of the given number.
     */
    ruleOfFour: (x: number) => string;
    /**
     * Given an array of values, returns the best formatter method
     *
     * @param arr  The values used to determine the optimal formatting method
     * @returns    The optimal formatting method
     */
    pickFormat: (arr: number[]) => any;
    /**
     * Formats a value as a float (exponential) using the rule of four to determine the precision.
     *
     * @param x  The number to format.
     * @param N  The precision.
     * @returns  A string with a formatted representation of the given number.
     */
    private _formatFloat(x, N?);
    /**
     * Given an array of values, returns statistics needed to determine the best formatter method
     *
     * @param arr  The values used to determine the optimal formatting method
     * @returns    The statistics needed to determine the best formatting method
     */
    private getStats(arr);
    /**
     * Given an number, returns the precision using the role of four
     *
     * @param x  The number.
     * @returns  A Number indicating the precision using the role of four
     */
    private getRofPrecision(x);
    /**
     * Determines if a value is an integer with the integer threshold
     *
     * @param x  The number.
     * @returns  A Boolean indicating whether or not the given value is an integer.
     */
    private isInteger(x);
}
export declare const ruleOfFour: (x: number) => string;
export declare const formatDecimal: (x: number) => string;
export declare const formatInteger: (x: number) => string;
export declare const formatFloat: (x: number) => string;
export declare const format: (x: number) => string;
export declare const pickFormat: (arr: number[]) => any;
